@@ -145,7 +145,7 @@ async fn main() -> Result<()> {
         // the name; used to represent it in plan descriptions and in the registry, to use in SQL.
         "geo_mean",
         // the input type; DataFusion guarantees that the first entry of `values` in `update` has this type.
-        DataType::Float64,
+        vec![DataType::Float64],
         // the return type; DataFusion expects this to match the type returned by `evaluate`.
         Arc::new(DataType::Float64),
         Volatility::Immutable,
