@@ -15,47 +15,49 @@
 // specific language governing permissions and limitations
 // under the License.
 
-pub mod alias;
-pub mod cast;
 mod column;
-pub mod config;
 mod dfschema;
-pub mod display;
 mod error;
-pub mod file_options;
-pub mod format;
 mod functional_dependencies;
-pub mod hash_utils;
 mod join_type;
-pub mod parsers;
 #[cfg(feature = "pyarrow")]
 mod pyarrow;
-pub mod scalar;
 mod schema_reference;
-pub mod stats;
 mod table_reference;
+mod unnest;
+
+pub mod alias;
+pub mod cast;
+pub mod config;
+pub mod display;
+pub mod file_options;
+pub mod format;
+pub mod hash_utils;
+pub mod parsers;
+pub mod scalar;
+pub mod stats;
 pub mod test_util;
 pub mod tree_node;
-mod unnest;
 pub mod utils;
 
+/// Reexport arrow crate
+pub use arrow;
 pub use column::Column;
 pub use dfschema::{DFField, DFSchema, DFSchemaRef, ExprSchema, SchemaExt, ToDFSchema};
 pub use error::{
     field_not_found, unqualified_field_not_found, DataFusionError, Result, SchemaError,
     SharedResult,
 };
-
 pub use file_options::file_type::{
     FileType, GetExt, DEFAULT_ARROW_EXTENSION, DEFAULT_AVRO_EXTENSION,
     DEFAULT_CSV_EXTENSION, DEFAULT_JSON_EXTENSION, DEFAULT_PARQUET_EXTENSION,
 };
 pub use file_options::FileTypeWriterOptions;
 pub use functional_dependencies::{
-    aggregate_functional_dependencies, get_target_functional_dependencies, Constraints,
-    Dependency, FunctionalDependence, FunctionalDependencies,
+    aggregate_functional_dependencies, get_target_functional_dependencies, Constraint,
+    Constraints, Dependency, FunctionalDependence, FunctionalDependencies,
 };
-pub use join_type::{JoinConstraint, JoinType};
+pub use join_type::{JoinConstraint, JoinSide, JoinType};
 pub use scalar::{ScalarType, ScalarValue};
 pub use schema_reference::{OwnedSchemaReference, SchemaReference};
 pub use stats::{ColumnStatistics, Statistics};
@@ -63,9 +65,6 @@ pub use table_reference::{OwnedTableReference, ResolvedTableReference, TableRefe
 pub use unnest::UnnestOptions;
 pub use utils::project_schema;
 
-/// Reexport arrow crate
-pub use arrow;
-
 /// Downcast an Arrow Array to a concrete type, return an `DataFusionError::Internal` if the cast is
 /// not possible. In normal usage of DataFusion the downcast should always succeed.
 ///
