@@ -16,22 +16,26 @@
 // under the License.
 
 //! listing_schema contains a SchemaProvider that scans ObjectStores for tables automatically
-use crate::catalog::schema::SchemaProvider;
-use crate::datasource::provider::TableProviderFactory;
-use crate::datasource::TableProvider;
-use crate::execution::context::SessionState;
-use async_trait::async_trait;
-use datafusion_common::parsers::CompressionTypeVariant;
-use datafusion_common::{DFSchema, DataFusionError, OwnedTableReference};
-use datafusion_expr::CreateExternalTable;
-use futures::TryStreamExt;
-use itertools::Itertools;
-use object_store::ObjectStore;
+
 use std::any::Any;
 use std::collections::{HashMap, HashSet};
 use std::path::Path;
 use std::sync::{Arc, Mutex};
 
+use crate::catalog::schema::SchemaProvider;
+use crate::datasource::provider::TableProviderFactory;
+use crate::datasource::TableProvider;
+use crate::execution::context::SessionState;
+
+use datafusion_common::parsers::CompressionTypeVariant;
+use datafusion_common::{Constraints, DFSchema, DataFusionError, OwnedTableReference};
+use datafusion_expr::CreateExternalTable;
+
+use async_trait::async_trait;
+use futures::TryStreamExt;
+use itertools::Itertools;
+use object_store::ObjectStore;
+
 /// A [`SchemaProvider`] that scans an [`ObjectStore`] to automatically discover tables
 ///
 /// A subfolder relationship is assumed, i.e. given:
@@ -149,6 +153,7 @@ impl ListingSchemaProvider {
                             order_exprs: vec![],
                             unbounded: false,
                             options: Default::default(),
+                            constraints: Constraints::empty(),
                         },
                     )
                     .await?;
