@@ -24,6 +24,7 @@ use arrow::datatypes::{
     DataType, Field, Fields, IntervalDayTimeType, IntervalMonthDayNanoType, IntervalUnit,
     Schema, SchemaRef, TimeUnit, UnionFields, UnionMode,
 };
+
 use prost::Message;
 
 use datafusion::datasource::provider::TableProviderFactory;
@@ -185,6 +186,94 @@ async fn roundtrip_custom_tables() -> Result<()> {
     Ok(())
 }
 
+#[tokio::test]
+async fn roundtrip_custom_memory_tables() -> Result<()> {
+    let ctx = SessionContext::new();
+    // Make sure during round-trip, constraint information is preserved
+    let query = "CREATE TABLE sales_global_with_pk (zip_code INT,
+          country VARCHAR(3),
+          sn INT,
+          ts TIMESTAMP,
+          currency VARCHAR(3),
+          amount FLOAT,
+          primary key(sn)
+        ) as VALUES
+          (0, 'GRC', 0, '2022-01-01 06:00:00'::timestamp, 'EUR', 30.0),
+          (1, 'FRA', 1, '2022-01-01 08:00:00'::timestamp, 'EUR', 50.0),
+          (1, 'TUR', 2, '2022-01-01 11:30:00'::timestamp, 'TRY', 75.0),
+          (1, 'FRA', 3, '2022-01-02 12:00:00'::timestamp, 'EUR', 200.0),
+          (1, 'TUR', 4, '2022-01-03 10:00:00'::timestamp, 'TRY', 100.0)";
+
+    let plan = ctx.sql(query).await?.into_optimized_plan()?;
+
+    let bytes = logical_plan_to_bytes(&plan)?;
+    let logical_round_trip = logical_plan_from_bytes(&bytes, &ctx)?;
+    assert_eq!(format!("{plan:?}"), format!("{logical_round_trip:?}"));
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn roundtrip_custom_listing_tables() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    // Make sure during round-trip, constraint information is preserved
+    let query = "CREATE EXTERNAL TABLE multiple_ordered_table_with_pk (
+              a0 INTEGER,
+              a INTEGER,
+              b INTEGER,
+              c INTEGER,
+              d INTEGER,
+              primary key(c)
+            )
+            STORED AS CSV
+            WITH HEADER ROW
+            WITH ORDER (a ASC, b ASC)
+            WITH ORDER (c ASC)
+            LOCATION '../core/tests/data/window_2.csv';";
+
+    let plan = ctx.sql(query).await?.into_optimized_plan()?;
+
+    let bytes = logical_plan_to_bytes(&plan)?;
+    let logical_round_trip = logical_plan_from_bytes(&bytes, &ctx)?;
+    assert_eq!(format!("{plan:?}"), format!("{logical_round_trip:?}"));
+
+    Ok(())
+}
+
+#[tokio::test]
+async fn roundtrip_logical_plan_aggregation_with_pk() -> Result<()> {
+    let ctx = SessionContext::new();
+
+    ctx.sql(
+        "CREATE EXTERNAL TABLE multiple_ordered_table_with_pk (
+              a0 INTEGER,
+              a INTEGER,
+              b INTEGER,
+              c INTEGER,
+              d INTEGER,
+              primary key(c)
+            )
+            STORED AS CSV
+            WITH HEADER ROW
+            WITH ORDER (a ASC, b ASC)
+            WITH ORDER (c ASC)
+            LOCATION '../core/tests/data/window_2.csv';",
+    )
+    .await?;
+
+    let query = "SELECT c, b, SUM(d)
+            FROM multiple_ordered_table_with_pk
+            GROUP BY c";
+    let plan = ctx.sql(query).await?.into_optimized_plan()?;
+
+    let bytes = logical_plan_to_bytes(&plan)?;
+    let logical_round_trip = logical_plan_from_bytes(&bytes, &ctx)?;
+    assert_eq!(format!("{plan:?}"), format!("{logical_round_trip:?}"));
+
+    Ok(())
+}
+
 #[tokio::test]
 async fn roundtrip_logical_plan_aggregation() -> Result<()> {
     let ctx = SessionContext::new();
@@ -424,59 +513,6 @@ fn scalar_values_error_serialization() {
             Some(vec![]),
             vec![Field::new("item", DataType::Int16, true)].into(),
         ),
-        // Should fail due to inconsistent types in the list
-        ScalarValue::new_list(
-            Some(vec![
-                ScalarValue::Int16(None),
-                ScalarValue::Float32(Some(32.0)),
-            ]),
-            DataType::List(new_arc_field("item", DataType::Int16, true)),
-        ),
-        ScalarValue::new_list(
-            Some(vec![
-                ScalarValue::Float32(None),
-                ScalarValue::Float32(Some(32.0)),
-            ]),
-            DataType::List(new_arc_field("item", DataType::Int16, true)),
-        ),
-        ScalarValue::new_list(
-            Some(vec![
-                ScalarValue::Float32(None),
-                ScalarValue::Float32(Some(32.0)),
-            ]),
-            DataType::Int16,
-        ),
-        ScalarValue::new_list(
-            Some(vec![
-                ScalarValue::new_list(
-                    None,
-                    DataType::List(new_arc_field("level2", DataType::Float32, true)),
-                ),
-                ScalarValue::new_list(
-                    Some(vec![
-                        ScalarValue::Float32(Some(-213.1)),
-                        ScalarValue::Float32(None),
-                        ScalarValue::Float32(Some(5.5)),
-                        ScalarValue::Float32(Some(2.0)),
-                        ScalarValue::Float32(Some(1.0)),
-                    ]),
-                    DataType::List(new_arc_field("level2", DataType::Float32, true)),
-                ),
-                ScalarValue::new_list(
-                    None,
-                    DataType::List(new_arc_field(
-                        "lists are typed inconsistently",
-                        DataType::Int16,
-                        true,
-                    )),
-                ),
-            ]),
-            DataType::List(new_arc_field(
-                "level1",
-                DataType::List(new_arc_field("level2", DataType::Float32, true)),
-                true,
-            )),
-        ),
     ];
 
     for test_case in should_fail_on_seralize.into_iter() {
@@ -511,7 +547,7 @@ fn round_trip_scalar_values() {
         ScalarValue::UInt64(None),
         ScalarValue::Utf8(None),
         ScalarValue::LargeUtf8(None),
-        ScalarValue::new_list(None, DataType::Boolean),
+        ScalarValue::List(ScalarValue::new_list(&[], &DataType::Boolean)),
         ScalarValue::Date32(None),
         ScalarValue::Boolean(Some(true)),
         ScalarValue::Boolean(Some(false)),
@@ -602,32 +638,32 @@ fn round_trip_scalar_values() {
             i64::MAX,
         ))),
         ScalarValue::IntervalMonthDayNano(None),
-        ScalarValue::new_list(
-            Some(vec![
+        ScalarValue::List(ScalarValue::new_list(
+            &[
                 ScalarValue::Float32(Some(-213.1)),
                 ScalarValue::Float32(None),
                 ScalarValue::Float32(Some(5.5)),
                 ScalarValue::Float32(Some(2.0)),
                 ScalarValue::Float32(Some(1.0)),
-            ]),
-            DataType::Float32,
-        ),
-        ScalarValue::new_list(
-            Some(vec![
-                ScalarValue::new_list(None, DataType::Float32),
-                ScalarValue::new_list(
-                    Some(vec![
+            ],
+            &DataType::Float32,
+        )),
+        ScalarValue::List(ScalarValue::new_list(
+            &[
+                ScalarValue::List(ScalarValue::new_list(&[], &DataType::Float32)),
+                ScalarValue::List(ScalarValue::new_list(
+                    &[
                         ScalarValue::Float32(Some(-213.1)),
                         ScalarValue::Float32(None),
                         ScalarValue::Float32(Some(5.5)),
                         ScalarValue::Float32(Some(2.0)),
                         ScalarValue::Float32(Some(1.0)),
-                    ]),
-                    DataType::Float32,
-                ),
-            ]),
-            DataType::List(new_arc_field("item", DataType::Float32, true)),
-        ),
+                    ],
+                    &DataType::Float32,
+                )),
+            ],
+            &DataType::List(new_arc_field("item", DataType::Float32, true)),
+        )),
         ScalarValue::Dictionary(
             Box::new(DataType::Int32),
             Box::new(ScalarValue::Utf8(Some("foo".into()))),
@@ -890,7 +926,6 @@ fn roundtrip_null_scalar_values() {
         ScalarValue::Date32(None),
         ScalarValue::TimestampMicrosecond(None, None),
         ScalarValue::TimestampNanosecond(None, None),
-        ScalarValue::List(None, Arc::new(Field::new("item", DataType::Boolean, false))),
     ];
 
     for test_case in test_types.into_iter() {
@@ -1112,7 +1147,17 @@ fn roundtrip_inlist() {
 
 #[test]
 fn roundtrip_wildcard() {
-    let test_expr = Expr::Wildcard;
+    let test_expr = Expr::Wildcard { qualifier: None };
+
+    let ctx = SessionContext::new();
+    roundtrip_expr_test(test_expr, ctx);
+}
+
+#[test]
+fn roundtrip_qualified_wildcard() {
+    let test_expr = Expr::Wildcard {
+        qualifier: Some("foo".into()),
+    };
 
     let ctx = SessionContext::new();
     roundtrip_expr_test(test_expr, ctx);
